%{
#include <blt.h>
#include <math.h>
%}

%include pointer.i

%inline %{
/* makeWaveForm -- constructs a waveform buffer for output to the ITC18
 * args: a: a buffer of shorts sufficient to contain all data channels written
 * samps: number of samples (per channel)
 * wfChoice: string containing the name of the waveform type
 * nChans: the number of channels
 * thisChan: which channel is being set, 0-based. This is necessary because
 *      channels are interleaved.
 * pipeline: size of the hardware pipeline (# samples lost per i/o session)
 */

void makeWaveForm(short *a, int samps, char *wfChoice, int nChans,
  int thisChan,
int pipeline) {

   int i;
   int begin = 20, amp=10000, sinePeriod = 50;
   int end = samps;

   if (thisChan < 0 || thisChan > (nChans - 1))
     thisChan = 0;
   if (strcmp(wfChoice, "Sine") == 0) {
      for (i = 0; i < samps + pipeline - begin; i++) {
	 a[(begin + i)*nChans + thisChan] =
	   (short)((double) amp*sin((double) i/(double)sinePeriod));
      }
      /* now set last points to 0 (leaving the pipeline clean) */
      a[(samps*nChans) + thisChan] = 0;
      a[((samps + 1)*nChans) + thisChan] = 0;
      a[((samps + 2)*nChans) + thisChan] = 0;
      
   } else {
      end = (int) ((float) samps * 0.75);
      for (i = 0; i < end; i++)
	a[(begin + i)*nChans + thisChan + 1] = 10000;
   }
}

/* arrayToVec -- converts an array into a BLT vector
 * Arguments
 * interp - pointer to the Tcl_Interp.  Necessary for BLT functions
 * array - pointer to the array of shorts collected by the hardware
 * vecName - name of the BLT vector already allocated
 * size - size of the array *** still necessary for error checking???
 * nChans - number of channels in the array, since the array is interleaved
 * thisChan - which channel to convert, 0-based
 * pipeline - number of bogus values to toss at the start and add at the end
 *    due to hardware pipeline
 */
int arrayToVec(Tcl_Interp *interp, short *array, char *vecName, int size,
    int nChans, int thisChan, int pipeline) {

  int i;
  Blt_Vector *theVec;

  if (Blt_VectorExists(interp, vecName)) {
    if (Blt_GetVector(interp, vecName, &theVec) != TCL_OK)
      return TCL_ERROR;
    } else {
      return TCL_ERROR;
  }
   
  /* should have the vector.  Now move values to the Vector array */
  for (i = 0; i < theVec->numValues; i++) {
    theVec->valueArr[i] = (double) array[nChans*i + thisChan];
  }
  
  Blt_ResetVector(theVec, theVec->valueArr, theVec->numValues,
    theVec->arraySize, TCL_STATIC);
  return TCL_OK;
}

%}
%{
static int wrapArrayToVec(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {

    int  _result;
    short * _arg0;
    char * _arg1;
    int  _arg2, _arg3, _arg4, _arg5;
    Tcl_Obj * tcl_result;
    char * rettype;
    int templength;
    int tempint;

    clientData = clientData; objv = objv;
    tcl_result = Tcl_GetObjResult(interp);
    if ((objc < 7) || (objc > 7)) {
        Tcl_SetStringObj(tcl_result,"Wrong # args. wrapArrayToVec array vecName size ",-1);
        return TCL_ERROR;
    }

    if ((rettype = SWIG_GetPointerObj(interp,objv[1],(void **) &_arg0,"_short_p"))) {
        Tcl_SetStringObj(tcl_result, "Type error in argument 1 of arrayToVec. Expected _short_p, received ", -1);
        Tcl_AppendToObj(tcl_result, rettype, -1);
        return TCL_ERROR;
    }
    if ((_arg1 = Tcl_GetStringFromObj(objv[2], &templength)) == NULL) return TCL_ERROR;
    if (Tcl_GetIntFromObj(interp,objv[3],&tempint) == TCL_ERROR) return TCL_ERROR;
    _arg2 = (int ) tempint;
    if (Tcl_GetIntFromObj(interp,objv[4],&tempint) == TCL_ERROR) return TCL_ERROR;
    _arg3 = (int ) tempint;
    if (Tcl_GetIntFromObj(interp,objv[5],&tempint) == TCL_ERROR) return TCL_ERROR;
    _arg4 = (int ) tempint;
    if (Tcl_GetIntFromObj(interp,objv[6],&tempint) == TCL_ERROR) return TCL_ERROR;
    _arg5 = (int ) tempint;

    _result = (int )arrayToVec(interp, _arg0, _arg1, _arg2, _arg3, _arg4, _arg5);
    tcl_result = Tcl_GetObjResult(interp);
    Tcl_SetIntObj(tcl_result,(long) _result);
    return TCL_OK;
}

%}


%init %{
  Tcl_PkgProvide(interp, "itc", "0.1");
  Tcl_CreateObjCommand(interp, SWIG_prefix "wrapArrayToVec", wrapArrayToVec, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
%}

